/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: frame.h,v 1.1.1.1 2003/08/13 06:11:01 maxk Exp $
 */

#ifndef _VAC_FRAME_H
#define _VAC_FRAME_H

struct vac_frame {
	uint8_t *data;		// Frame buffer
	int      size;		// Buffer size
	int      length;	// Actual length of the frame
	int      refcnt;

	int      width;
	int      height;
	int      pixfmt;
	uint8_t *plane[4];
	int      hline[4];
	int      vline[4];
};

static inline struct vac_frame *vac_frame_alloc(int size)
{
	struct vac_frame *f = malloc(size + sizeof(*f));
	if (f) {
		f->data   = (void *) f + sizeof(*f);
		f->size   = size;
		f->refcnt = 1;
	}
	return f;
}	

static inline struct vac_frame *vac_frame_get(struct vac_frame *f)
{
	f->refcnt++;
	return f;
}

static inline void vac_frame_put(struct vac_frame *f)
{
	if (--f->refcnt == 0)
		free(f);
}

int vac_frame_fill(struct vac_frame *f,
		int pix, int width, int height);

int vac_frame_get_length(int pix, int width, int height, int *length);

#endif /* _VAC_FRAME_H */
