/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: input.h,v 1.1.1.1 2003/08/13 06:11:01 maxk Exp $
 */

#ifndef _VAC_INPUT_H
#define _VAC_INPUT_H

#define VAC_MAX_INPUTS 30

struct vac_input;

struct vac_input_param {
	int    width;
	int    height;
	float  frate;
	char  *format;

	struct vac_input *diff[2];
};

struct vac_input_ops {
	char *format;
	char *ext;
	char *desc;

	int  (*open)  (struct vac_input *vi);
	int  (*read)  (struct vac_input *vi, struct vac_frame **f);
	int  (*seek)  (struct vac_input *vi, int frameno);
	int  (*close) (struct vac_input *vi);

	struct vac_input_ops *next;
};

int  vac_register_input(struct vac_input_ops *ops);
int  vac_unregister_input(struct vac_input_ops *ops);

struct vac_input {
	char  *location;
	int    index;

	int    width;
	int    height;
	int    total_frames;
	int    pixfmt;

	float  actual_frate;
	float  target_frate;

	int    frameno;
	struct vac_frame *frame;

	float  psnr[4];
	struct vac_input *diff[2];

	void  *priv;
	void  *out;
	struct vac_input_ops *ops;
};

int  vac_input_open(char *location, struct vac_input_param *p, struct vac_input **inp);
int  vac_input_close(struct vac_input *vi);
int  vac_input_read(struct vac_input *vi, struct vac_frame **f);
int  vac_input_seek(struct vac_input *vi, int frameno);

struct vac_input_group {
	struct vac_input *input[VAC_MAX_INPUTS];
	int    num;

	int    err_fsize; 
	int    err_frate;
	int    err_total;
};

int  vac_input_add(struct vac_input_group *grp, char *location, struct vac_input_param *p);
int  vac_input_del(struct vac_input_group *grp, struct vac_input *in);
void vac_input_validate(struct vac_input_group *grp);

#endif /* _VAC_INPUT_H */
