/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: pixel.h,v 1.1.1.1 2003/08/13 06:11:01 maxk Exp $
 */

#ifndef _VAC_PIXEL_H
#define _VAC_PIXEL_H

/* 
 * Pixel conversion and frame initialization routines are based on
 *	FFMPEG Misc image convertion routines
 *	Copyright (c) 2001, 2002, 2003 Fabrice Bellard.
 */

enum {
	VAC_PIX_YUV422,
	VAC_PIX_YUV420P,
	VAC_PIX_YUV422P,
	VAC_PIX_YUV444P,
	VAC_PIX_YUV410P,
	VAC_PIX_YUV411P,
	VAC_PIX_RGB24,
	VAC_PIX_BGR24,
	VAC_PIX_RGBA32,
	VAC_PIX_RGB565,
	VAC_PIX_RGB555
};

enum {
	VAC_COLOR_RGB,       /* RGB color space */
	VAC_COLOR_YUV,       /* YUV color space. 16 <= Y <= 235, 16 <= U, V <= 240 */
};

enum {
	VAC_PIX_PLANAR,      /* each channel has one component in AVPicture */
	VAC_PIX_PACKED,      /* only one components containing all the channels */
	VAC_PIX_PALETTE,     /* one components containing indexes for a palette */
};

struct vac_pix_format {
	const char *name;
	uint8_t channels;        /* number of channels (including alpha) */
	uint8_t color_type;      /* color type (see VAC_COLOR_xxx constants) */
	uint8_t pixel_type;      /* pixel storage type (see VAX_PIX_xxx constants) */
	uint8_t is_alpha;        /* true if alpha can be specified */
	uint8_t x_chroma_shift;  /* X chroma subsampling factor is 2 ^ shift */
	uint8_t y_chroma_shift;  /* Y chroma subsampling factor is 2 ^ shift */
	uint8_t depth;           /* bit depth of the color components */
};
extern struct vac_pix_format vac_pixfmt_table[];

int vac_pix_info(int pix, int kind);

#endif /* _VAC_PIXEL_H */
