/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: diff.c,v 1.1.1.1 2003/08/13 06:10:55 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <errno.h>
#include <math.h>

#include <sys/stat.h>

#include <vac/vacum.h>

static inline void __diff(struct vac_frame *df[], struct vac_frame *of)
{
	int i;

	/* Y */
	for (i = 0; i < df[0]->hline[0] * df[0]->vline[0]; i++) {
		int d = abs((df[0]->plane[0][i] - df[1]->plane[0][i]));
		if (d >= 5)
			d += 50;
		of->plane[0][i] = d;
	}

	/* U */
	for (i = 0; i < df[0]->hline[1] * df[1]->vline[1]; i++)
		of->plane[1][i] = 128 + (df[0]->plane[1][i] - df[1]->plane[1][i]);

	/* V */
	for (i = 0; i < df[0]->hline[2] * df[1]->vline[2]; i++)
		of->plane[2][i] = 128 + (df[0]->plane[2][i] - df[1]->plane[2][i]);
}

static int diff_read(struct vac_input *vi, struct vac_frame **f)
{
	struct vac_frame *frame, *df[2];
	int i;

	df[0] = vi->diff[0]->frame;
	df[1] = vi->diff[1]->frame;

	if (!df[0] || !df[1])
		return VAC_ERR_EOF;

	frame = vac_frame_alloc(df[0]->size);
	if (!frame)
		return -1;
	vac_frame_fill(frame, VAC_PIX_YUV420P, vi->width, vi->height);

	for (i=0; i < 3; i++)
		vi->psnr[i] = vac_psnr(df[0]->plane[i], df[1]->plane[i],
				df[0]->hline[i] * df[0]->vline[i]);

	__diff(df, frame);

	*f = frame;
	
	return 0;
}

static int diff_seek(struct vac_input *vi, int frame)
{
	return 0;
}

static int diff_open(struct vac_input *vi)
{
	vi->width  = vi->diff[0]->width;
	vi->height = vi->diff[0]->height;

	vi->total_frames = vi->diff[0]->total_frames;
	return 0;
}

static int diff_close(struct vac_input *vi)
{
	return 0;
}

static struct vac_input_ops diff_input_ops = {
	.format = "diff",
	.ext    = "diff",
	.desc   = "Visual diff format",
	.open   = diff_open,
	.close  = diff_close,
	.seek   = diff_seek,
	.read   = diff_read,
};

void diff_init(void)
{
	vac_register_input(&diff_input_ops);
}

void diff_deinit(void)
{
	vac_unregister_input(&diff_input_ops);
}
