/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: group.c,v 1.1.1.1 2003/08/13 06:10:55 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>

#include <vac/vacum.h>

/* Input collections (groups) */

int vac_input_add(struct vac_input_group *grp, char *location, struct vac_input_param *vip)
{
	struct vac_input *in;
	int err;

	if (grp->num >= VAC_MAX_INPUTS)
		return VAC_ERR_INPUT_MAX;

	err = vac_input_open(location, vip, &in);
	if (err)
		return err;

	grp->input[grp->num++] = in;
	return 0;
}

void vac_input_validate(struct vac_input_group *grp)
{
	int prev_width = 0, prev_height = 0;
	int prev_total = 0;
	float max_frate = 0;
	int i;

	grp->err_frate = 0;
	grp->err_fsize = 0;

	vac_info("Inputs:");
	for (i=0; i < grp->num; i++) {
		struct vac_input *in = grp->input[i];

		in->index = i;

		vac_info("%d: %s\n"
			"\tFrame size: %dx%d\tFrame rate: %.2f\n"
			"\tNumber of frames: %d",
				in->index, in->location,
				in->width, in->height,
				in->actual_frate, in->total_frames);

		if (in->actual_frate > max_frate) {
			if (max_frate)
				grp->err_frate++;
			max_frate = in->actual_frate;
		}

		if ((prev_width && in->width != prev_width) ||
				(prev_height && in->height != prev_height))
			grp->err_fsize++;
		prev_width  = in->width;
		prev_height = in->height;

		if (prev_total && in->total_frames != prev_total)
			grp->err_total++;
		prev_total  = in->total_frames;
	}
	vac_info("Target frame rate: %.2f\n", max_frate);

	if (grp->err_frate) {
		/* Adjust target frame rate for all inputs in the group */
		for (i=0; i < grp->num; i++) {
			struct vac_input *in = grp->input[i];
			in->target_frate = max_frate;
		}
	}
}
