/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: interface.c,v 1.1.1.1 2003/08/13 06:10:56 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include <vac/vacum.h>

#include "display.h"

#define INTERFACE_FILE "vacplay.glade"

static char *interface_file = INTERFACE_FILE;

static GtkStatusbar *statusbar;
static GtkAdjustment *adjust;

struct info_box {
	GtkWidget *frameno;
	GtkWidget *psnr;
};

struct info_box *infobox;

void ui_set_status(char *str)
{
	gtk_statusbar_pop(statusbar, 0);
	gtk_statusbar_push(statusbar, 0, str);
}

void ui_set_scroll(int pos)
{
	adjust->value = pos;
	gtk_adjustment_changed(adjust);
}

void ui_update_info(struct vac_input *in)
{
	int n = in->index;
	char str[50];

	sprintf(str, "%.4d", in->frameno);
	gtk_label_set_text(GTK_LABEL(infobox[n].frameno), str);

	sprintf(str, "%4.2f", in->psnr[0]);
	gtk_label_set_text(GTK_LABEL(infobox[n].psnr), str);
}

static int create_info_boxes(GtkWidget *parent, struct vac_input_group *ins, int row)
{
	GtkWidget *frame, *lbox = NULL, *vbox, *hbox, *label;
	char str[10], *ptr;
	int i;

	for (i=0; i < ins->num; i++) {
		if (!(i % row)) {
			lbox = gtk_hbox_new(FALSE, 0);
			gtk_box_pack_start(GTK_BOX(parent), lbox, TRUE, TRUE, 0);
			gtk_widget_show(lbox);
		}
	
		ptr = strrchr(ins->input[i]->location, '/');
		if (!ptr)
			ptr = ins->input[i]->location;
		else
			ptr++;

		sprintf(str, "%d: %s", i, ptr);
		frame = gtk_frame_new(str);
		gtk_widget_set_usize(GTK_WIDGET(frame), 150, 50);
		gtk_box_pack_start(GTK_BOX(lbox), frame, TRUE, TRUE, 0);

		vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(frame), vbox);

		/* FRAMENO */
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

		label = gtk_label_new("Frame: ");
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_widget_show(label);

		label = gtk_label_new("0000");
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_widget_show(label);
		infobox[i].frameno = label;
		
		gtk_widget_show(hbox);

		/* PSNR */
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

		label = gtk_label_new("PSNR: ");
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_widget_show(label);

		label = gtk_label_new("0000");
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_widget_show(label);
		infobox[i].psnr = label;
		
		gtk_widget_show(hbox);

		gtk_widget_show(vbox);
		gtk_widget_show(frame);
	}

	return 0;
}

void on_scrollbar_value_changed(GtkWidget *widget, gpointer user_data)
{
	display_scroll((int) adjust->value);
}

void on_play_button_pressed(GtkWidget *widget, gpointer user_data)
{
	display_playback();
}

void on_stop_button_pressed(GtkWidget *widget, gpointer user_data)
{
	display_stop();
}

void on_next_button_pressed(GtkWidget *widget, gpointer user_data)
{
	display_frame(NULL);
}

void on_about_activate(GtkWidget *widget, gpointer user_data)
{
	GladeXML *xml;
	GtkWidget *w;

	/* load the interface */
	xml = glade_xml_new(interface_file, "about_dialog", NULL);
	if (!xml) {
		vac_perror(-1, "Failed to load interface file %s", interface_file);
		return;
	}

	/* connect the signals in the interface */
	glade_xml_signal_autoconnect(xml);
	
	w = glade_xml_get_widget(xml, "about_dialog");
	if (!w) {
		vac_perror(-1, "About dialog not found");
		return;
	}

	gtk_dialog_run(GTK_DIALOG(w));
	gtk_widget_destroy(w);

	g_object_unref(G_OBJECT(xml));
}

int ui_init(int x, int y, struct vac_input_group *ins, int row)
{
	GladeXML *xml;
	GtkWidget *w;

	if (!gtk_init_check(NULL, NULL)) {
		vac_perror(-1, "GTK initialization failed");
		return -1;
	}

	if (access(interface_file, R_OK) < 0)
		interface_file = PKGDATADIR "/" INTERFACE_FILE;
		
	/* load the interface */
	xml = glade_xml_new(interface_file, "main_window", NULL);
	if (!xml) {
		vac_perror(-1, "Failed to load interface file %s", interface_file);
		return -1;
	}

	w = glade_xml_get_widget(xml, "main_window");
	if (w)
		gtk_widget_set_uposition(w, x, y);

	/* connect the signals in the interface */
	glade_xml_signal_autoconnect(xml);

	infobox = malloc(sizeof(struct info_box) * ins->num);
	if (!infobox) {
		vac_perror(-1, "Frame info allocation failed");
		return -1;
	}

	w = glade_xml_get_widget(xml, "infopanel");
	if (!w) {
		vac_perror(-1, "Infopannel widget not found");
		return -1;
	}
	create_info_boxes(w, ins, row);

	w = glade_xml_get_widget(xml, "scrollbar");
	if (!w) {
		vac_perror(-1, "Scrollbar widget not found");
		return -1;
	}
	adjust = gtk_range_get_adjustment(GTK_RANGE(w));

	adjust->lower = 0;
	adjust->value = 0;
	adjust->upper = ins->input[0]->total_frames - 1;
	adjust->step_increment = 1;
	adjust->page_increment = 10;
	adjust->page_size = 0;
	gtk_adjustment_changed(adjust);

	statusbar = GTK_STATUSBAR(glade_xml_get_widget(xml, "statusbar"));
	gtk_statusbar_push(statusbar, 0, "Ready");
	return 0;
}

void ui_loop(void)
{	
	gtk_main();
}
