/* 
   VACUM - Video Analysis and Comparison Utilities
   Copyright (C) 2003 Max Krasnyansky <maxk@qualcomm.com>

   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
   version 2 as published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
   NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE 
   COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM, OR 
   ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
   ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF 
   THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY 
   PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO
   USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * $Id: vout.c,v 1.1.1.1 2003/08/13 06:10:56 maxk Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <math.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/uio.h>

#include <SDL/SDL.h>

#include <vac/vacum.h>
#include "vout.h"

struct vout {
	SDL_Overlay *overlay;
	SDL_Rect     rect;
};

static const SDL_VideoInfo *vi;
static SDL_Surface *surface;

int vout_create(int width, int height)
{
	//putenv("SDL_VIDEO_YUV_HWACCEL=1");
	//putenv("SDL_VIDEO_X11_NODIRECTCOLOR=1");

	if (SDL_Init(SDL_INIT_VIDEO)) {
		vac_perror(-1, "SDL initialization failed");
		return -1;
	}

	vi = SDL_GetVideoInfo();

	surface = SDL_SetVideoMode(width, height, 24, SDL_HWSURFACE);
	if (!surface) {
		vac_perror(-1, "Surface creation failed");
		return -1;
	}

	SDL_WM_SetCaption("VACUM display", NULL);
	return 0;
}

int vout_destroy(void)
{
	SDL_FreeSurface(surface);
	SDL_QuitSubSystem(SDL_INIT_VIDEO);
	return 0;
}

int vout_open(struct vout_param *p, struct vout **vout)
{
	struct vout *vo;

	vo = malloc(sizeof(*vo));
	if (!vo)
		return -1;
	memset(vo, 0, sizeof(*vo));

	vo->rect    = (SDL_Rect) {p->x, p->y, p->width, p->height};

	vo->overlay = SDL_CreateYUVOverlay(p->width, p->height, SDL_YV12_OVERLAY, surface);
	if (!vo->overlay) {
		vac_perror(-1, "Video overlay creation failed");
		goto failed;
	}

	*vout = vo;
	return 0;

failed:
	vout_close(vo);
	return -1;
}

int vout_close(struct vout *vo)
{
	if (vo->overlay)
		SDL_FreeYUVOverlay(vo->overlay);
	free(vo);
	return 0;
}

int vout_display(struct vout *vo, struct vac_frame *f)
{
	SDL_Overlay *ov = vo->overlay;

	SDL_LockYUVOverlay(ov);

	/* Y */
	memcpy(ov->pixels[0], f->plane[0], f->hline[0] * f->vline[0]);

	/* U */
	memcpy(ov->pixels[2], f->plane[1], f->hline[1] * f->vline[1]);

	/* V */
	memcpy(ov->pixels[1], f->plane[2], f->hline[2] * f->vline[2]);

	SDL_UnlockYUVOverlay(ov);

	SDL_DisplayYUVOverlay(ov, &vo->rect);

	return 0;
}
